-- SOI 2015 Creativity Task 'Bazaar'
-- Sample bot in Haskell
import Data.List (sort, group)
import System.Random (RandomGen, newStdGen, randomRs, split)

makeBets :: RandomGen rng => Int -> Int -> rng -> [Int]
makeBets gold diamonds = map (subtract 1) . map length . group . sort . (++[0..(diamonds-1)]) . take gold . randomRs (0, diamonds-1)

bid :: Int -> [Int] -> [Int] -> Int
bid round balances bets = bets!!round

playAuction :: RandomGen rng => (Int, Int, Int) -> rng -> [String] -> ([Int], [String])
playAuction (n, d, g) gen ls = playRounds d (replicate n 0) ls
    where
      bets = makeBets g d gen
      updateBalances b = zipWith (+) b . map read . words
      playRounds 0 _ ls = ([], ls)
      playRounds i b ls = let (bids', ls') = playRounds (i-1) (updateBalances b (head ls)) (init ls)
                          in ((bid (d-i) b bets):bids', ls')

play :: RandomGen rng => rng -> [String] -> [String]
play gen (l:ls) = map show . concat . map snd . take (a+1) . iterate go $ ((gen, ls), [])
    where
      [n, a, d, g] = map read . words $ l
      go ((gen, ls), _) = let (gen', gen'') = split gen
                              (out, ls') = playAuction (n, d, g) gen' ls
                            in ((gen'', ls'), out)

main = newStdGen >>= \gen -> interact (unlines . play gen . lines)
